#ifndef MYX_REDIS_PUBSUB_HPP_
#define MYX_REDIS_PUBSUB_HPP_

#pragma once

#include <myx/redis/base.hpp>

#include <QObject>

QT_FORWARD_DECLARE_CLASS( QTcpSocket )

namespace myx {

namespace redis {

/*!
 * \brief УПравляет подпиской и публикацией сообщений
 */

class PubSub : public Base
{
	Q_OBJECT

public:
	PubSub( QObject* parent = nullptr );
	bool isConnected() const;
	QTcpSocket* socket() const;
	Q_SLOT void start();
	Q_SLOT void stop();
	Q_INVOKABLE void subscribe( QString channel, QObject* subscriber, const char* method );
	Q_INVOKABLE void subscribe( QString channel, QObject* subscriber,
	                            const char* method, int database );
	Q_INVOKABLE void psubscribe( QString channel, QObject* subscriber, const char* method );
	Q_INVOKABLE void psubscribe( QString channel, QObject* subscriber,
	                             const char* method, int database );
	Q_INVOKABLE void unsubscribe( QString channel, QObject* subscriber, const char* method );
	Q_INVOKABLE void unsubscribe( QString channel, QObject* subscriber,
	                              const char* method, int database );
	Q_INVOKABLE void punsubscribe( QString channel, QObject* subscriber, const char* method );
	Q_INVOKABLE void punsubscribe( QString channel, QObject* subscriber,
	                               const char* method, int database );
	/*!
	 * \brief Сигнализирует об установке соединения
	 */
	Q_SIGNAL void connected();
	/*!
	 * \brief Сигнализирует о разрыве соединения
	 */
	Q_SIGNAL void disconnected();

private:
	using Sub = QHash< QString, std::vector< std::pair< QObject*, const char* > > >;

	static const QString KeySpacePrefix; //!< Приставка для наблюдения за данными
	QTcpSocket*          _socket;
	/*!
	 * \brief Таблица подписчиков
	 */
	QByteArray _buffer;                  //!< Буффер прочитанных данных
	Sub        _subscribers;             //!< Обработчики сообщений по каналу подписки
	Sub        _psubscribers;            //!< Обработчики сообщений по шаблону канала подписки

	Q_SLOT void read();
	Q_SLOT void onSocketStateChanged();
}; // class PubSub

} // namespace redis

} // namespace myx

#ifdef MYXLIB_HEADER_ONLY
#include "pubsub-inl.hpp"
#endif

#endif // ifndef MYX_REDIS_PUBSUB_HPP_
